@@ -1,9 +1,9 @@
 package Test::Smoke::Reporter;
 use strict;
 
-# $Id: Reporter.pm 1037 2007-04-02 21:17:19Z abeltje $
+# $Id: Reporter.pm 1176 2008-05-03 00:26:15Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.027';
+$VERSION = '0.033';
 
 use Cwd;
 use File::Spec::Functions;
@@ -15,18 +15,22 @@ use Test::Smoke::Util qw( grepccmsg get_smoked_Config
                           time_in_hhmm get_local_patches );
 
 my %CONFIG = (
-    df_ddir       => curdir(),
-    df_outfile    => 'mktest.out',
-    df_rptfile    => 'mktest.rpt',
-    df_cfg        => undef,
-    df_lfile      => undef,
-    df_showcfg    => 0,
+    df_ddir         => curdir(),
+    df_outfile      => 'mktest.out',
+    df_rptfile      => 'mktest.rpt',
+    df_cfg          => undef,
+    df_lfile        => undef,
+    df_showcfg      => 0,
 
-    df_locale     => undef,
-    df_defaultenv => undef,
-    df_is56x      => undef,
+    df_locale       => undef,
+    df_defaultenv   => undef,
+    df_is56x        => undef,
+    df_skip_tests   => undef,
 
-    df_v          => 0,
+    df_harnessonly  => undef,
+    df_harness3opts => undef,
+
+    df_v            => 0,
 );
 
 =head1 NAME
@@ -300,6 +304,11 @@ sub _parse {
                 if ref $rpt{$cfgarg}->{$debug}{$tstenv};
             next;
         }
+        if ( /^\s+(?:Bad plan)|(?:No plan found)/ ) {
+            push @{ $rpt{$cfgarg}->{$debug}{$tstenv} }, $_
+                if ref $rpt{$cfgarg}->{$debug}{$tstenv};
+            next;
+        }
         next;
     }
 
@@ -322,7 +331,7 @@ sub _post_process {
 
     unless ( defined $self->{is56x} ) {
         my %cfg = get_smoked_Config( $self->{ddir}, 'version' );
-        my $p_version = sprintf "%d.%03d%03d", split /\./, $cfg{version};
+        my $p_version = sprintf "%d.%03d%03d", split m/\./, $cfg{version};
         $self->{is56x} = $p_version < 5.007;
     }
     $self->{defaultenv} ||= $self->{is56x};
@@ -363,7 +372,10 @@ sub _post_process {
                 next if $tstenv eq 'minitest' && ! exists $status->{ $tstenv };
 
                 ( my $showenv = $tstenv ) =~ s/^locale://;
-                $self->{_locale} ||= $showenv if $tstenv =~ /^locale:/;
+                if ( $tstenv =~ /^locale:/ ) {
+                    $self->{_locale_keys}{ $showenv }++
+                        or push @{ $self->{_locale} }, $showenv;
+                }
                 $showenv = 'default' 
                     if $self->{defaultenv} && $showenv eq 'stdio';
 
@@ -390,9 +402,11 @@ sub _post_process {
             }
             unless ( $self->{defaultenv} ) {
                 exists $status->{perlio} or $status->{perlio} = '-';
-                exists $status->{ "locale:$self->{locale}" } or 
-                    $status->{ "locale:$self->{locale}" } = '-'
-                        if $self->{locale};
+                my @locales = split ' ', ($self->{locale} || '');
+                for my $locale ( @locales ) {
+                    exists $status->{ "locale:$locale" } or 
+                        $status->{ "locale:$locale" } = '-'
+                }
             }
 
             $count{ $_ }++ for map {
@@ -400,6 +414,8 @@ sub _post_process {
             } map $status->{ $_ } => keys %$status;
         }
     }
+    defined $self->{_locale} or $self->{_locale} = [ ];
+
     my @failures = map {
         { tests => $_,
           cfgs  => [ map {
@@ -488,6 +504,10 @@ sub report {
 
     $report .= $self->registered_patches;
 
+    $report .= $self->harness3_options;
+
+    $report .= $self->user_skipped_tests;
+
     $report .= "\nFailures: (common-args) $self->{_rpt}{common_args}\n"
             .  $self->failures if $self->has_test_failures;
     $report .= "\n" . $self->mani_fail           if $self->has_mani_failures;
@@ -538,7 +558,43 @@ sub registered_patches {
     @lpatches or return "";
 
     my $list = join "\n", map "    $_" => @lpatches;
-    return "Locally applied patches:\n$list\n";
+    return "\nLocally applied patches:\n$list\n";
+}
+
+=item $reporter->harness3_options
+
+Show indication of the options used for C<HARNESS_OPTIONS>.
+
+=cut
+
+sub harness3_options {
+    my $self = shift;
+
+    $self->{harnessonly} or return "";
+
+    my $msg = "\nTestsuite was run only with 'harness'";
+    $self->{harness3opts} or return $msg . "\n";
+
+    return  $msg . " and HARNESS_OPTIONS=$self->{harness3opts}\n";
+}
+
+=item $reporter->user_skipped_tests( )
+
+Show indication for the fact that the user requested to skip some tests.
+
+=cut
+
+sub user_skipped_tests {
+    my( $self ) = @_;
+    $self->{skip_tests} && -f $self->{skip_tests} or return "";
+
+    local *NOTESTS;
+    open NOTESTS, "< $self->{skip_tests}" or return "";
+
+    my $skipped = join "\n", map { chomp; "    $_" } <NOTESTS>;
+    close NOTESTS;
+
+    return "\nTests skipped on user request:\n$skipped";
 }
 
 =item $reporter->ccmessages( )
@@ -580,7 +636,10 @@ sub preamble {
     ));
     my $si = Test::Smoke::SysInfo->new;
     my $archname  = $si->cpu_type;
-    $archname .= sprintf "/%s cpu", $si->ncpu if $si->ncpu;
+ 
+    (my $ncpu = $si->ncpu || "") =~ s/^(\d+)\s*/$1 cpu/;
+    $archname .= "/$ncpu";
+
     my $cpu = $si->cpu;
 
     my $this_host = $si->host;
@@ -704,11 +763,38 @@ Returns a string with the legend for build-environments
 
 sub bldenv_legend {
     my $self = shift;
-    my $locale = $self->{_locale};
     $self->{defaultenv} = ( @{ $self->{_tstenv} } == 1 )
         unless defined $self->{defaultenv};
     my $debugging = $self->{_rpt}{dbughow} || '-DDEBUGGING';
 
+    if ( $self->{_locale} && @{ $self->{_locale} } ) {
+        my @locale = ( @{ $self->{_locale} }, @{ $self->{_locale} } );
+        my $lcnt = @locale;
+        my $half = int(( 4 +  $lcnt ) / 2 );
+        my $cnt = 2 * $half;
+
+        my $line = '';
+        for my $i ( 0 .. $cnt-1 ) {
+            $line .= '| ' x ( $cnt - 1 - $i );
+            $line .= '+';
+            $line .= '-' x (2 * $i);
+            $line .= '- ';
+
+            if ( ($i % $half) < ($lcnt / 2) ) {
+                my $locale = shift @locale;     # XXX: perhaps pop()
+                $line .= "LC_ALL = $locale"
+            } else {
+                $line .= ( (($i - @{$self->{_locale}}) % $half) % 2 == 0 )
+                    ? "PERLIO = perlio"
+                    : "PERLIO = stdio ";
+            }
+            $i < $half and $line .= " $debugging";
+            $line .= "\n";
+        }
+        return $line;
+    }
+
+    my $locale = ''; # XXX
     return  $locale ? <<EOL : $self->{defaultenv} ? <<EOS : <<EOE;
 | | | | | +- LC_ALL = $locale $debugging
 | | | | +--- PERLIO = perlio $debugging
